import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Serializable;
import java.io.Writer;
import java.util.Date;
import java.util.Properties;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.destatis.core.inspector.InspectionReport;
import de.destatis.core.inspector.Inspector;
import de.destatis.core.resbuilder.ResBuilder;
import de.destatis.prueftool.gwa.exceptions.GWAPrueftoolException;
import de.destatis.prueftool.gwa.util.InitializeLogger;


/**
 * <p><b>&Uuml;berschrift:</b> GWAPrueftool</p>
 * <p><b>Beschreibung:</b> Die Klasse GWAPrueftool dient zur Durchf&uuml;hrung des Pr&uuml;fprozesses einer Datenlieferung und der darin enthaltenen 
 *                         Gewerbeanzeige auf formale Richtigkeit und zur Erzeugung der gewerbeanzeigespezifischen Protokolldatei (DatML/RES) 
 *                         mit ggfls. enthaltenen Fehlern.<br>
 *                         <br>
 *                         Nachfolgende vom Statistischen Bundesamt bereitgestellten Komponente sind im GWA-Pr&uuml;ftool enthalten:
 *                         <ul>
 *                         <li>
 *                         <i>CORE.inspector</i><br>
 *                         <div style="padding-left:30px;">
 *                         Java Paket, das eine DatML/RAW-Datei (Gewerbeanzeige) auf formale, syntaktische und fachliche Fehler &uuml;berpr&uuml;fen kann<br>
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>ResBuilder</i><br>
 *                         <div style="padding-left:30px;">
 *                         Java Paket, das zur Erstellung der Protokolldatei im Format DatML/RES dient<br>
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>Sylesheet</i><br>
 *                         <div style="padding-left:30px;">
 *                         Transformationsscript zur Anpassung der verfahrensspezifischen Besonderheiten der Protokolldatei zur Gewerbeanzeige<br>
 *                         </div>
 *                         </li>
 *                         </ul>
 *                         <br>
 *                         <img src="gwa_prueftool.jpg" alt="&Uuml;bersicht GWAPr&uuml;ftool" width="40%" height="40%"><br>
 *                         Abbildung: Darstellung des Pr&uuml;fablaufs bei Einsatz des GWA-Pr&uuml;ftools<br>
 *                         <br>
 *                         <br>
 *                         Der CORE.inspector ben&ouml;tigt zur Validierung einer Datenlieferung eine Definition der formalen und fachlichen Abh&auml;ngigkeiten 
 *                         innerhalb einer Gewerbeanzeige. Diese Definition wird in Form einer XML-Datei im Format DatML/SDF zur Verf&uuml;gung gestellt. 
 *                         Dieses Format geh&ouml;rt zur DatML-Familie und wird ebenfalls durch das Statistische Bundesamt bereitgestellt.<br>
 *                         Die Spezifikation des Formate DatML/SDF findet man &uuml;ber das Erhebungsportal der Statistik unter https://erhebungsportal.estatistik.de 
 *                         <img src="arrow-right.png" alt="->" width="8px" height="8px"> Hilfsmittel und Automatisierung 
 *                         <img src="arrow-right.png" alt="->" width="8px" height="8px"> Unterst&uuml;tzung f&uuml;r Entwickler 
 *                         <img src="arrow-right.png" alt="->" width="8px" height="8px"> Spezifikationen zu CORE 
 *                         <img src="arrow-right.png" alt="->" width="8px" height="8px"> Datenformate 
 *                         <img src="arrow-right.png" alt="->" width="8px" height="8px"> <u>Artikel:</u> 'Das Format DatML/SDF'.<br> 
 *                         <br>
 *                         <br>
 *                         
 *                         
 *<b>Parameterbeschreibung:</b><br>
 *                         <ul>
 *                         <li style="padding-top:6px;">
 *                         <i>inputDir</i> - Das Eingabeverzeichnis der XSTATISTIK-Meldungen als relativer Pfad.<br>
 *                         <div style="padding-left:30px;">
 *                         Beim Aufruf der Erzeugung der gewerbeanzeigespezifischen Protokolldatei (Methode <b><i>generateProtokoll(...)</i></b>) ist das Eingabeverzeichnis 
 *                         der zu pr&uuml;fenden XSTATISTIK-Meldungen als Paramterer zu &uuml;bergeben.<br>
 *                         Der angegebene relative Pfad f&uuml;r das Eingabeverzeichnis muss einem g&uuml;ltigen Verzeichnisnamen entsprechen und vorhandenen sein.<br>       
 *                         Als Default-Wert wird das <b><i>"in"</i></b>-Verzeichnis verwendet.<br>                                                
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>inputFileXML</i> - Der Dateiname der zu pr&uuml;fenden XSTATISTIK-Meldung.<br>
 *                         <div style="padding-left:30px;">
 *                         Beim Aufruf der Erzeugung der gewerbeanzeigespezifischen Protokolldatei (Methode <b><i>generateProtokoll(...)</i></b>) ist der Dateiname der 
 *                         zu pr&uuml;fenden XSTATISTIK-Meldung als Paramterer zu &uuml;bergeben.<br>
 *                         Der angegebene Dateiname muss einem g&uuml;ltigen Dateinamen entsprechen und im Eingabeverzeichnis vorhandenen sein.<br>                                                
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>outputDir</i> - Das Ausgabeverzeichnis der zu erzeugenden Protokolldatei als relativer Pfad.<br>
 *                         <div style="padding-left:30px;">
 *                         Das Ausgabeverzeichnis kann mittels der Methode <b><i>setOutputDir(String outputDir)</i></b> angepasst werden.<br>
 *                         Der angegebene relative Pfad f&uuml;r das Ausgabeverzeichnis muss einem g&uuml;ltigen Verzeichnisnamen entsprechen und vorhandenen sein.<br>       
 *                         Als Default-Wert wird das <b><i>"out"</i></b>-Verzeichnis verwendet.<br>                                                
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>outputResFile</i> - Der Dateiname der zu erzeugenden Protokolldatei.<br>
 *                         <div style="padding-left:30px;">
 *                         Der Protokolldateiname kann mittels der Methode <b><i>setOutputResFile(String outputResFile)</i></b> angepasst werden. Nach Erzeugung 
 *                         der Protokolldatei wird diese im Ausgabeverzeichnis abgelegt.<br>
 *                         Als Default-Wert wird der Dateiname der zu pr&uuml;fenden XSTATISTIK-Meldung (ohne Extention '.xml') erweitert um den Anhang "-prot.xml"
 *                         verwendet.<br>
 *                         Beispiel: <span style="color:darkblue;"><i>DATEINAME.xml</i></span> (XSTATISTIK-Meldung) <img src="arrow-right.png" alt="->" width="8px" height="8px">
 *                         <span style="color:darkblue;"><i>DATEINAME-prot.xml</i></span> (Protokolldatei)</div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>eingangsstempel</i> - Der Eingangsstempel kann zur eindeutigen Kennzeichnung des Dateneingangs der XSTATISTIK-Meldung dienen.<br>
 *                         <div style="padding-left:30px;">
 *                         Der Eingangsstempel zur Hinterlegung im Dateneingang der Protokolldatei kann mittels der Methode <b><i>setEingangsstempel(String eingangsstempel)</i></b>
 *                         angepasst werden.<br>
 *                         Der Eingangsstempel ist zur eindeutigen Kennzeichnung des Dateneingangs der XSTATISTIK-Meldung und der Protokolldatei f&uuml;r die weitere 
 *                         Verarbeitung in den internen Prozessen der Statistik erforderlich.<br>
 *                         Als Default-Wert wird der Dateiname der zu pr&uuml;fenden XSTATISTIK-Meldung verwendet.<br> 
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>logsDir</i> - Das Verzeichnis der Log-Dateien als relativer Pfad.<br>
 *                         <div style="padding-left:30px;">
 *                         Das Verzeichnis der Log-Dateien kann mittels der Methode <b><i>setLogsDir(String logsDir)</i></b> angepasst werden.<br> 
 *                         Der angegebene relative Pfad f&uuml;r das Verzeichnis der Log-Dateien muss einem g&uuml;ltigen Verzeichnisnamen entsprechen und 
 *                         vorhandenen sein.<br>       
 *                         Als Default-Wert wird das <b><i>"logs"</i></b>-Verzeichnis verwendet.<br>                                                
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>surveyDir</i> - Das Erhebungsbeschreibungsverzeichnis zur Hinterlegung der erforderlichen Erhebungsbeschreibungen 
 *                         als relativer Pfad.<br>
 *                         <div style="padding-left:30px;">
 *                         Das Erhebungsbeschreibungsverzeichnis kann mittels der Methode <b><i>setSurveyDir(String surveyDir)</i></b> angepasst werden.<br>
 *                         Der angegebene relative Pfad f&uuml;r das Erhebungsbeschreibungsverzeichnis muss einem g&uuml;ltigen Verzeichnisnamen entsprechen und vorhandenen sein.<br>       
 *                         Als Default-Wert wird das <b><i>"sdf"</i></b>-Verzeichnis verwendet.<br>                                                
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>xsltDir</i> - Das XSLT-Verzeichnis zur Ablage des Transformationsscripts (res2res-gewerbe.xsl) und der dazugeh&ouml;rigen Konfigurationsdatei 
 *                         (res2res-gewerbe.config.xml) als relativer Pfad.<br>
 *                         <div style="padding-left:30px;">
 *                         Das XSLT-Verzeichnis kann mittels der Methode <b><i>setXsltDir(String xsltDir)</i></b> angepasst werden.<br> 
 *                         Der angegebene relative Pfad f&uuml;r das XSLT-Verzeichnis muss einem g&uuml;ltigen Verzeichnisnamen entsprechen und vorhandenen sein.<br>       
 *                         Als Default-Wert wird das <b><i>"xslt"</i></b>-Verzeichnis verwendet.<br>                                                
 *                         </div>
 *                         </li>
 *                         <li style="padding-top:6px;">
 *                         <i>inspectionLevel</i> - Gibt die Pr&uuml;fstufe des CORE.inspectors an.<br>
 *                         <div style="padding-left:30px;">
 *                         Die Pr&uuml;fstufe (InspectionLevel) kann mittels der Methode <b><i> 	setInspectionLevel(int inspectionLevel)</i></b> angepasst werden.<br> 
 *                         g&uuml;ltige Werte sind:<br>
 *                         <ul>
 *                         <li>"3" - "Pr&uuml;fstufe 3 - Autorisierung: Einhaltung allgemeiner, nicht statistikspezifischer Vorgaben zur Datenlieferung mit DatML/RAW."</li>
 *                         <li>"4" - "Pr&uuml;fstufe 4 - Daten: Einhaltung der Vorgaben lt. der zur jeweiligen Meldung geh&ouml;renden Erhebungsbeschreibung, also z.B. 
 *                         Anzahl und Art der anzugebenden Merkmale und Merkmalsgruppen, Angabe des Berichtszeitraums und der Erhebung."</li> 
 *                         </ul>
 *                         Als Default-Wert wird die Pr&uuml;fstufe <b><i>"4"</i></b> verwendet.<br>                                                
 *                         </div>
 *                         </li>
 *                         </ul>
 *                         <br>
 *                         <br>
 *                         
 *                         
 *<b>Systemvoraussetzungen:</b><br>
 *                         <div style="padding-left:30px;">
 *                         Das GWAPrueftool ist auf jeder Hardwareplattform einsetzbar, f&uuml;r welche eine Java-Laufzeitumgebung (JRE) ab Version 1.7 zur Verf&uuml;gung steht.
 *                         </div>
 *                         <br>
 *                         <br>
 *                         
 *                         
 *<b>Erforderliche Anpassungen:</b><br>
 *                         <ul>
 *                         <li style="padding-top:6px;">
 *                         Zur Anpassung der Beschreibung der Anwendung, welche die gewerbeanzeigespezifische Protokolldatei (DatML/RES) mittels des GWA-Pr&uuml;ftools erzeugt, 
 *                         muss in der Konfigurationsdatei '<b>res2res-gewerbe.config.xml</b>' (befindet sich im XSLT-Verzeichnis) der <i><b>Anwendungsname</b></i>, die dazugeh&ouml;rige 
 *                         <i><b>Versionsbezeichnung</b></i> der Anwendung und der <i><b>Hersteller</b></i> der Anwendung modifiziert werden.<br>   
 *                         <div style="padding-left:20px;padding-top:6px;">
 *                         <i>
 *                         &lt;anwendung&gt;<br>
 *                         &nbsp;&nbsp;&nbsp;&lt;name&gt;<font color="darkred"><b>ANWENDUNGSNAME</b></font>&lt;/name&gt;<br>
 *                         &nbsp;&nbsp;&nbsp;&lt;version&gt;<font color="darkred"><b>VERSION</b></font>&lt;/version&gt;<br>
 *                         &nbsp;&nbsp;&nbsp;&lt;hersteller&gt;<font color="darkred"><b>HERSTELLER</b></font>&lt;/hersteller&gt;<br>
 *                         &lt;/anwendung&gt;<br>
 *                         </i>
 *                         </div>
 *                         <div style="padding-top:6px;">
 *                         <u>Hinweis:</u>Wird in der Konfigurationsdatei ein leeres Element hinterlegt, so wird dieses Element nicht bercksichtigt.<br>  
 *                         &nbsp;&nbsp;&nbsp;z.B.:&nbsp;&nbsp;&lt;name&gt;&lt;/name&gt;&nbsp;-&nbsp;Keine Anpassung des Namens<br>
 *                         </div>
 *                         </li>
 *                         </ul>
 *                         <br>
 *                         <br>
 *                         
 *                         
 *<b>Beispiel eines Aufrufs zur Pr&uuml;fung der XStatistik-Meldung 'BeispielSpezifikation.xml':</b><br>
 *                         <div style="padding-left:30px;">
 *			               GWAPrueftool tool = new GWAPrueftool();<br>
 *                         </div> 
 *                         <div style="padding-left:30px;">
 *                         tool.generateProtokoll("BeispielSpezifikation.xml");<br>
 *                         </div> 
 *                         <br>
 * </p> 
 * 
 * 
 * @author Statistisches Bundesamt Wiesbaden
 * 
 * @version V1.0
 * 
 * @since 24.03.2015
 * 
 */
public class GWAPrueftool_Example implements Serializable {
	private static final long serialVersionUID = 1L;
	
	private int inspectionLevel = 4;

	private String defaultInputDir = null;
	private String defaultOutputDir = null;
	private String defaultLogsDir = null;
	private String defaultSurveyDir = null;
	private String defaultXsltDir = null;
	
	private String inputDir = null;
	private String outputDir = null;
	private String logsDir = null;
	private String surveyDir = null;
	private String xsltDir = null;
	
	private static Properties properties = null;
	private String inputXMLFile = null;
	private String eingangsstempelToUse = null;
	private String outputResFile = null;
	private String outputResFileToDelete = null;
	private final String xsltFileName = "res2res-gewerbe.xsl";
	private final String configFileName = "res2res-gewerbe.config.xml";
	
	/**
	 * Aufruf
	 */
	public static void main(String[] args) {
		GWAPrueftool_Example tool = new GWAPrueftool_Example();
		try {
			tool.generateProtokoll("BeispielSpezifikation.xml");
		} catch (IOException e) {
			e.printStackTrace();
		} catch (GWAPrueftoolException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
    }
	
	/**
	 * Konstruktor. Erzeugt eine neue Instanz des GWAPrueftool zur Pr&uuml;fung der XSTATISTIK-Meldung
	 * mittels des CORE.inspectors unter Verwendung der Erhebungsbeschreibung, zur Erzeugung der 
	 * Protokolldatei im Format DatML/RES und der Durchf&uuml;hrung der verfahrensspezifischen 
	 * Anpassungen der Protokolldatei mittels des Transformationsscripts zur Gewerbeanzeige.
	 * 
	 * @throws IOException 
	 */
	public GWAPrueftool_Example() {
		super();
		
		defaultInputDir = "in";
		defaultOutputDir = "out";
		defaultLogsDir = "logs";
		defaultSurveyDir = "sdf";
		defaultXsltDir = "xslt";
		
		inputDir = defaultInputDir;
		outputDir = defaultOutputDir;
		logsDir = defaultLogsDir;
		surveyDir = defaultSurveyDir;
		xsltDir = defaultXsltDir;
	}

	private void setProperties(String propertiesFile) throws IOException {
		properties = new Properties();

		FileInputStream in = null;
		try {
			in = new FileInputStream(propertiesFile);
			properties.load(in);
			in.close();
		} catch (Exception e) {
			if (in!=null) 
			    in.close();
			throw e;
		}
		
		defaultInputDir  = properties.getProperty("input.dir");
		defaultOutputDir = properties.getProperty("output.dir");
		defaultLogsDir   = properties.getProperty("logs.dir");
		defaultSurveyDir = properties.getProperty("survey.dir");
		defaultXsltDir   = properties.getProperty("xslt.dir");

		inputDir  = defaultInputDir;
		outputDir = defaultOutputDir;
		logsDir   = defaultLogsDir;
		surveyDir = defaultSurveyDir;
		xsltDir   = defaultXsltDir;
	}
	
	/**
	 * Dient zur Erzeugung der gewerbeanzeigespezifischen Protokolldatei (DatML/RES) zur &uuml;bergebenen und zu pr&uuml;fenden XSTATISTIK-Meldung.
	 * 
     * @param inputFileXML Die zu pr&uuml;fende XSTATISTIK-Meldung. 
     * 
	 * @throws IOException falls eine IOException mit der zu pr&uuml;fenden XSTATISTIK-Meldung oder der Protokolldatei auftritt.
     * @throws GWAPrueftoolException falls das Eingabeverzeichnis "in" nicht existiert, der Dateiname der XSTATISTIK-Meldung mit <code>null</code> angegeben 
     *                               wurde oder die angegebene zu pr&uuml;fende XSTATISTIK-Meldung nicht existiert.
	 * @throws ParserConfigurationException 
	 * @throws SAXException 
	 * @throws TransformerConfigurationException 
	 */
	public void generateProtokoll(String inputFileXML) throws IOException, GWAPrueftoolException, TransformerException, SAXException, ParserConfigurationException {
		generateProtokoll(defaultInputDir, inputFileXML, inputFileXML);
	}
	
	/**
	 * Dient zur Erzeugung der gewerbeanzeigespezifischen Protokolldatei (DatML/RES) zur &uuml;bergebenen und zu pr&uuml;fenden XSTATISTIK-Meldung.
	 * 
     * @param inputFileXML Die zu pr&uuml;fende XSTATISTIK-Meldung. 
     * @param propertiesFile Die Properties-Datei mit der Konfiguration zu den Verzeichnissen.
     * 
	 * @throws IOException falls eine IOException mit der zu pr&uuml;fenden XSTATISTIK-Meldung oder der Protokolldatei auftritt.
     * @throws GWAPrueftoolException falls das Eingabeverzeichnis "in" nicht existiert, der Dateiname der XSTATISTIK-Meldung mit <code>null</code> angegeben 
     *                               wurde, die angegebene zu pr&uuml;fende XSTATISTIK-Meldung oder die angegebene Properties-Datei nicht existiert .
	 * @throws ParserConfigurationException 
	 * @throws SAXException 
	 * @throws TransformerConfigurationException 
	 */
	public void generateProtokollWithProperties(String inputFileXML, String propertiesFile) throws IOException, GWAPrueftoolException, TransformerException, SAXException, ParserConfigurationException {
		setProperties(propertiesFile);
		generateProtokoll(defaultInputDir, inputFileXML, inputFileXML);
	}
	
	/**
	 * Dient zur Erzeugung der gewerbeanzeigespezifischen Protokolldatei (DatML/RES) zur &uuml;bergebenen und zu pr&uuml;fenden XSTATISTIK-Meldung .
	 * 
     * @param inputDir Das Eingabeverzeichnis der XSTATISTIK-Meldungen. Falls <code>null</code> angegeben, wird der Default-Wert "in" verwendet.
     * @param inputFileXML Die zu pr&uuml;fende XSTATISTIK-Meldung. 
     * 
	 * @throws IOException falls eine IOException mit der zu pr&uuml;fenden XSTATISTIK-Meldung oder der Protokolldatei auftritt.
     * @throws GWAPrueftoolException falls das Eingabeverzeichnis nicht existiert, der Dateiname der XSTATISTIK-Meldung mit <code>null</code> angegeben 
     *                               wurde oder die angegebene zu pr&uuml;fende XSTATISTIK-Meldung nicht existiert.
	 * @throws TransformerException 
	 * @throws ParserConfigurationException 
	 * @throws SAXException 
	 */
	public void generateProtokoll(String inputDir, String inputFileXML) throws IOException, GWAPrueftoolException, TransformerException, SAXException, ParserConfigurationException {
		generateProtokoll(inputDir, inputFileXML, inputFileXML);
	}
	
	/**
	 * Dient zur Erzeugung der gewerbeanzeigespezifischen Protokolldatei (DatML/RES) zur &uuml;bergebenen und zu pr&uuml;fenden XSTATISTIK-Meldung .
	 * 
     * @param inputDir Das Eingabeverzeichnis der XSTATISTIK-Meldungen. Falls <code>null</code> angegeben, wird der Default-Wert "in" verwendet.
     * @param inputFileXML Die zu pr&uuml;fende XSTATISTIK-Meldung. 
     * @param eingangsstempel Der Eingangsstempel zur Hinterlegung im Dateneingang der Protokolldatei. Falls <code>null</code> angegeben, 
     *                        wird der Dateiname der XSTATISTIK-Meldung als Eingangsstempel verwendet.
     * 
	 * @throws IOException falls eine IOException mit der zu pr&uuml;fenden XSTATISTIK-Meldung oder der Protokolldatei auftritt.
     * @throws GWAPrueftoolException falls das Eingabeverzeichnis nicht existiert, der Dateiname der XSTATISTIK-Meldung mit null angegeben 
     *                               wurde oder die angegebene zu pr&uuml;fende XSTATISTIK-Meldung nicht existiert.
	 * @throws TransformerException 
	 * @throws ParserConfigurationException 
	 * @throws SAXException 
	 */
	public void generateProtokoll(String inputDir, String inputFileXML, String eingangsstempel) throws IOException, GWAPrueftoolException, TransformerException, SAXException, ParserConfigurationException {
		new InitializeLogger(logsDir+File.separator+"GWAPrueftool.log");
		
		if (inputDir==null)
			this.inputDir=defaultInputDir;
		else
			this.inputDir = inputDir;
			
		if(!(new File(this.inputDir)).exists())
			throw new GWAPrueftoolException("Das Eingabeverzeichnis '"+this.inputDir+"' muss einem g\u00FCltigem Pfad entsprechen.");
			
		if (inputFileXML == null)
			throw new GWAPrueftoolException("Der Dateinamen der XSTATISTIK-Meldung darf nicht null sein.");

		inputXMLFile = inputFileXML;
		if (! new File(this.inputDir+"/"+inputXMLFile).exists())
			throw new GWAPrueftoolException("Der Dateinamen der XSTATISTIK-Meldung '"+inputFileXML+"' existiert nicht im Eingabeverzeichnis '"+this.inputDir+"'.");

		if (eingangsstempel==null)
			eingangsstempelToUse = new File(inputXMLFile).getName();
		else
			eingangsstempelToUse = eingangsstempel;

		if (outputResFile==null)
			outputResFile = inputFileXML.substring(0, inputFileXML.length()-4)+"-prot.xml";
		outputResFileToDelete = outputResFile+"-ToDelete";
		
		InspectionReport report = inspect();
		
		generateProtokoll(report);
		
		transform();
	}
	
	/**
	 * Liefert den aktuell eingestellten InspectionLevel.
	 * 
	 * @return int des aktuell eingestellten InspectionLevels
	 */
	public int getInspectionLevel() {
		return inspectionLevel;
	}
	/**
	 * Setzt den InspectionLevel.
	 * 
     * @param inspectionLevel Der f&uuml;r die Pr&uuml;fung zu verwendende InspectionLevel.
     * <p>
     * g&uuml;ltige Werte sind:<br>
     * <ul>
     * <li>"3" - "Pr&uuml;fstufe 3 - Autorisierung: Einhaltung allgemeiner, nicht statistikspezifischer Vorgaben zur Datenlieferung mit DatML/RAW."</li>
     * <li>"4" - "Pr&uuml;fstufe 4 - Daten: Einhaltung der Vorgaben lt. der zur jeweiligen Meldung geh&ouml;renden Erhebungsbeschreibung, also z.B. Anzahl und Art der anzugebenden Merkmale und Merkmalsgruppen, Angabe des Berichtszeitraums und der Erhebung."</li> 
     * </ul>
     * </p>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Default-Wert: '4'<br>
     * 
     * @throws GWAPrueftoolException falls eine Pr&uuml;fstufe ungleich 3 (Pr&uuml;fstufe Autorisierung) oder 4 (Pr&uuml;fstufe Daten) angegeben wurde.
	 */
	public void setInspectionLevel(int inspectionLevel) throws GWAPrueftoolException {
		if (inspectionLevel < 3 && inspectionLevel > 5)
			throw new GWAPrueftoolException("Die angegebene Pr\u00FCfstufe entspricht keinem g\u00FCltigem Wert. Es sind nur die Pr\u00FCfstufen 3 oder 4 zul\u00E4ssig.");
		this.inspectionLevel = inspectionLevel;
	}

	/**
	 * Liefert das aktuell eingestellte Ausgabeverzeichnis zur Ablage der erzeugten Protokolldateien.
	 * 
	 * @return String des Ausgabeverzeichnisses
	 */
	public String getOutputDir() {
		return outputDir;
	}
	/**
	 * Setzt das Ausgabeverzeichnis zur Ablage der erzeugten Protokolldateien.
	 * 
     * @param outputDir Das Ausgabeverzeichnis zur Ablage der erzeugten Protokolldateien. Das Ausgabeverzeichnis muss existieren und 
     *                  darf nicht <code>null</code> sein.<br>
     * <p>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Default-Wert: "out"<br>
     * </p>
     * 
     * @throws GWAPrueftoolException falls das Ausgabeverzeichnis mit null angegeben wurde oder keinem g\u00FCltigem Pfad entspricht.
	 */
	public void setOutputDir(String outputDir) throws GWAPrueftoolException {
		if (outputDir == null)
			throw new GWAPrueftoolException("Das Ausgabeverzeichnis darf nicht null sein.");
		else if (!(new File(outputDir)).exists())
			throw new GWAPrueftoolException("Das Ausgabeverzeichnis '"+outputDir+"' muss einem g\u00FCltigem Pfad entsprechen.");
		
		this.outputDir = outputDir.replace("\\", "/");
	}

	/**
	 * Liefert das aktuell eingestellte Logs-Verzeichnis.
	 * 
	 * @return String des Logs-Verzeichnis
	 */
	public String getLogsDir() {
		return logsDir;
	}
	/**
	 * Setzt das Logs-Verzeichnis.
	 * 
     * @param logsDir Das Logs-Verzeichnis. Das Logs-Verzeichnis muss existieren und darf nicht <code>null</code> sein.<br>
     * <p>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Default-Wert: "logs"<br>
     * </p>
     * 
     * @throws GWAPrueftoolException falls das Logs-Verzeichnis mit null angegeben wurde oder keinem g\u00FCltigem Pfad entspricht.
	 */
	public void setLogsDir(String logsDir) throws GWAPrueftoolException {
		if (logsDir == null)
			throw new GWAPrueftoolException("Das Logs-Verzeichnis darf nicht null sein.");
		else if (!(new File(logsDir)).exists())
			throw new GWAPrueftoolException("Das Logs-Verzeichnis muss einem g\u00FCltigem Pfad entsprechen.");
		
		this.logsDir = logsDir.replace("\\", "/");
	}

	/**
	 * Liefert das aktuell eingestellte Erhebungsbeschreibungsverzeichnis.
	 * 
	 * @return String des Erhebungsbeschreibungsverzeichnis
	 */
	public String getSurveyDir() {
		return surveyDir;
	}
	/**
	 * Setzt das Erhebungsbeschreibungsverzeichnis.
	 * 
     * @param surveyDir Das Erhebungsbeschreibungsverzeichnis. Das Erhebungsbeschreibungsverzeichnis muss existieren und darf nicht <code>null</code> sein.<br>
     * <p>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Default-Wert: "sdf"<br>
     * </p>
     * 
     * @throws GWAPrueftoolException falls das Erhebungsbeschreibungsverzeichnis mit null angegeben wurde oder keinem g\u00FCltigem Pfad entspricht.
	 */
	public void setSurveyDir(String surveyDir) throws GWAPrueftoolException {
		if (surveyDir == null)
			throw new GWAPrueftoolException("Das Erhebungsbeschreibungsverzeichnis darf nicht null sein.");
		else if (!(new File(surveyDir)).exists())
			throw new GWAPrueftoolException("Das Erhebungsbeschreibungsverzeichnis muss einem g\u00FCltigem Pfad entsprechen.");
		
		this.surveyDir = surveyDir.replace("\\", "/");
	}

	/**
	 * Liefert das aktuell eingestellte Verzeichnis der Transformationsdatei (res2res-gewerbe.xsl) und der dazugeh&ouml;rigen 
	 * Konfigurationsdatei (res2res-gewerbe.config.xml).<p>
	 * In der Konfigurationsdatei (res2res-gewerbe.config.xml) ist die Beschreibung der Anwendung anhand des Anwenungsnamens, der dazugeh&ouml;rigen Version
     * und des Herstellers vorzunehmen:<br>
     * <ul>
     * <li>'name' durch den Anwendungsnamen</li>
     * <li>'version' durch die Versionsbezeichnung der Anwendung</li> 
     * <li>'hersteller' durch den Hersteller der Anwendung</li> 
     * </ul>
	 * 
	 * @return String des Erhebungsbeschreibungsverzeichnis
	 */
	public String getXsltDir() {
		return xsltDir;
	}
	/**
	 * Setzt das Verzeichnis der Transformationsdatei (res2res-gewerbe.xsl) und der dazugeh&ouml;rigen Konfigurationsdatei (res2res-gewerbe.config.xml).<p>
	 * In der Konfigurationsdatei ist die Beschreibung der Anwendung anhand des Anwenungsnamens, der dazugeh&ouml;rigen Version
     * und des Herstellers vorzunehmen:<br>
     * <ul>
     * <li>'name' durch den Anwendungsnamen</li>
     * <li>'version' durch die Versionsbezeichnung der Anwendung</li> 
     * <li>'hersteller' durch den Hersteller der Anwendung</li> 
     * </ul>
	 * 
     * @param xsltDir Das Verzeichnis der Transformationsdatei und der dazugeh&ouml;rigen Konfigurationsdatei. Das Verzeichnis der Transformationsdatei und 
     * der dazugeh&ouml;rigen Konfigurationsdatei muss existieren und darf nicht <code>null</code> sein.
     * <p>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Default-Wert: "xslt"<br>
     * </p>
     * 
     * @throws GWAPrueftoolException falls das Verzeichnis der Transformationsdatei und der dazugeh\u00F6rigen Konfigurationsdatei mit null angegeben wurde oder keinem g\u00FCltigem Pfad entspricht.
	 */
	public void setXsltDir(String xsltDir) throws GWAPrueftoolException {
		if (xsltDir == null)
			throw new GWAPrueftoolException("Das Verzeichnis der Transformationsdatei und der dazugeh\u00F6rigen Konfigurationsdatei darf nicht null sein.");
		else if (!(new File(xsltDir)).exists())
			throw new GWAPrueftoolException("Das Verzeichnis der Transformationsdatei und der dazugeh\u00F6rigen Konfigurationsdatei muss einem g\u00FCltigem Pfad entsprechen.");
		
		this.xsltDir = xsltDir.replace("\\", "/");
	}

	/**
	 * Liefert den aktuell eingestellten Dateinamen der Protokolldatei.
	 * 
	 * @return String des Dateinamens der Protokolldatei
	 */
	public String getOutputResFile() {
		return outputResFile;
	}
	/**
	 * Setzt den Dateinamen der Protokolldatei.
	 * 
     * @param outputResFile Der Dateinamen der Protokolldatei. Die Protokolldatei darf nicht <code>null</code> sein.
     * <p>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Default-Wert: Dateiname der zu pr&uuml;fenden XSTATISTIK-Meldung (ohne Extention '.xml') erweitert um den Anhang "-prot.xml".
     * </p>
     * 
     * @throws GWAPrueftoolException falls die Protokolldatei mit null oder eine ung&uuml;ltiger Dateiname angegeben wurde.
	 */
	public void setOutputResFile(String outputResFile) throws GWAPrueftoolException {
		if (outputResFile == null)
			throw new GWAPrueftoolException("Die Dateiname der Protokolldatei darf nicht null sein.");
		else if (outputResFile.indexOf("\\") >= 0 ||
				 outputResFile.indexOf("/") >= 0)
			throw new GWAPrueftoolException("Es muss ein g&uuml;ltiger Dateiname ohne Verzeichnisstruktur angegeben werden.");
		
		this.outputResFile = outputResFile;
	}

	private InspectionReport inspect() throws IOException {
		//
		// Zunaechst den Inspectionreports mittels des CORE.inspectors erzeugen
		//
		Inspector inspector = new Inspector();

        // das aktuell eingestellte Erhebungsbeschreibungsverzeichnis des Inspectors setzen
        inspector.setSurveyDir(getSurveyDir());

        // Legt die vom Inspector verwendete Pruefstufe 1 - 4 fest.
        // Pruefstufe 1 - Syntax: Wohlgeformtheit und Gueltigkeit gemae XML-Format DatML/RAW.
        // Pruefstufe 2 - Semantik: Einhaltung aller weiteren in der DatML/RAW-Spezifikation festgelegten Formatvorgaben.
        // Pruefstufe 3 - Autorisierung: Einhaltung allgemeiner, nicht statistikspezifischer Vorgaben zur Datenlieferung mit DatML/RAW.
        // Pruefstufe 4 - Daten: Einhaltung der Vorgaben lt. der zur jeweiligen Meldung gehoerenden Erhebungsbeschreibung, also z.B. Anzahl und Art der anzugebenden Merkmale und Merkmalsgruppen, Angabe des Berichtszeitraums und der Erhebung.
        inspector.setInspectionLevel(inspectionLevel);
        
        // Ueberpruefung starten
        InputSource inputSource = new InputSource(new FileInputStream(inputDir+"/"+inputXMLFile));
        InspectionReport report = inspector.inspect(inputSource);
        return report;
	}
	
	private void generateProtokoll(InspectionReport report) throws IOException {
		//
        // Mittels des ResBuilders unter Verwendung des zurvor erzeugten Inspectionreports die Protokolldatei im Format DatML/RES erzeugen
        //
        ResBuilder rb = new ResBuilder();
        FileReader fileReader = new FileReader(inputDir+"/"+inputXMLFile);
        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(getOutputDir()+"/"+outputResFileToDelete), "UTF-8"));

        // report - der Report aus dem Inspectormodul, das die Prueffehler und Konfiguration enthaelt
        // fileReader - der Reader, aus dem das DatML/RAW-Dokument gelesen wird
        // bw - der Writer, in das das Pruefprotokoll geschrieben wird als DatML/RES
        // eingangsstempel - das Transferticket, das zur Ermittlung des Dateneinganges (Datum, Uhrzeit, Kennung) dient
        // input - der Dateiname, unter dem das Rawdokument uebermittelt wurde
        // new Date() - das Lieferdatum der zugehoerigen Datenlieferung
        rb.writeResProtocol(report, fileReader, bw, eingangsstempelToUse, inputDir+"/"+inputXMLFile, new Date());

        bw.flush();
        bw.close();
        fileReader.close();
	}
	
	private void transform() throws IOException, TransformerException, SAXException, ParserConfigurationException {
		//
		// Mittels Stylesheet die Anpassung der verfahrensspezifischen Besonderheiten der Protokolldatei zur Gewerbeanzeige durchfuehren
		//

		// Stylesheet
		Source xsltSource = new StreamSource(new File(xsltDir+"/"+xsltFileName)); 
		// zu transformierendes Pruefprotokoll
		Source transformSource = new StreamSource(new File(outputDir+"/"+outputResFileToDelete));
		
		TransformerFactory transformerFactory = TransformerFactory.newInstance("net.sf.saxon.TransformerFactoryImpl", null);
		Templates templates = transformerFactory.newTemplates(xsltSource);
		Transformer transformer = templates.newTransformer();

		transformer.setParameter("datml-raw-doc", new File(inputDir+"/"+inputXMLFile).toURI());
		transformer.setParameter("config-file", new File(xsltDir+"/"+configFileName).toURI());

		Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outputDir+"/"+outputResFile), "UTF-8")); 
		transformer.transform(transformSource, new StreamResult(writer));			
		writer.close();

		new File(outputDir+"/"+outputResFileToDelete).deleteOnExit();
	}
}