<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xpath-default-namespace="http://www.destatis.de/schema/datml-raw/2.0/de"
    xmlns="http://www.destatis.de/schema/datml-raw/2.0/de"
    xmlns:func="http://www.destatis.de/schema/datml-raw/2.0/de/functions"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    exclude-result-prefixes="xs"
    version="2.0"
    >
    
    <!-- 
     **********************************************************************
        
     Stylesheet : res2res-gewerbe
     Modul      : res2res-gewerbe
     Version    : 0.4
     Variante   : -
     Typ        : XSLT (.xsl)
     Stand      : 2015-10-19
     Autor      : Michael Schäfer, Statistisches Bundesamt
     
     Parameter:
     ==========
     
     datml-raw-doc
                  URI des geprüften DatML/RAW Dokuments
     config-file
                  URI der Konfigurationsdatei für dieses Stylesheet
                
     Beschreibung:
     =============
     
     Ein DatML/RES Dokument wird in vier Phasen mittels Templates mit einem
     der folgenden Modi transformiert:
     
     - 'flag'
       Warnungen werden entsprechend der Konfiguration zu Fehlern propagiert.
     - 'message'
       Die Stati der Meldungen werden angepasst
     - 'document'
       Die Stati des Dokuments und der Nachrichten werden angepasst, und es
       werden UUIDs gesetzt.
     - 'text'
       Textersetzungen werden durchgeführt.
       
     Die Verarbeitung findet auch statt, wenn das zugehörige DatML/RAW-
     Dokument nicht gelesen oder verarbeitet werden kann. In diesem Fall
     werden keine UUIDs gesetzt, da diese aus der Meldung gelesen werden.
     
     Version 0.3
       - Berichtseinheit-Id wird in Meldung ausgegeben

     Version 0.4
       - nachfolgende Anpassungen wurden Aufgrund der möglichen Übergabe von 
         Streams vorgenommen: 
       		- Prüfung auf Dateiexistenz deaktiviert
       		- Variablen 'datml-raw-doc-intern' und 'config-file-intern' aufgenommen
          
     **********************************************************************
     -->
    
    <xsl:strip-space elements="*"/>
    <xsl:output indent="yes"/>
        
    <xsl:param name="datml-raw-doc" required="yes"/>
    <xsl:variable name="datml-raw-doc-intern" select="$datml-raw-doc"/>
    <xsl:param name="config-file" required="yes"/>
    <xsl:variable name="config-file-intern" select="$config-file"/>
    
    <!-- Zugriff auf Dokumente prüfen
         Für den Aufruf aus Java (Übergabe von Streams) ist die Prüfung delativiert
        <xsl:variable name="raw-exists" as="xs:boolean" select="func:checkDocument($datml-raw-doc,'DatML/RAW Meldung')"/>
        <xsl:variable name="cfg-exists" as="xs:boolean" select="func:checkDocument($config-file,'Konfigurationsdatei')"/>
    -->
    
    <xsl:variable name="raw-exists" as="xs:boolean" select="true()"/>
    <xsl:variable name="cfg-exists" as="xs:boolean" select="true()"/>
    
    <!-- Auswertbarkeit des DatML/RAW-Dokumentes prüfen (es muss syntaktisch korrekt sein) -->
    
    <xsl:variable name="document-processible" as="xs:boolean" select="boolean(/DatML-RES-D/pruefprotokoll/pruefung/syntax[@pruefstatus eq 'OK'])"/>
    
    <!-- Verarbeitungsmodus ermitteln -->
    
    <xsl:variable name="processingMode" select="if (/DatML-RES-D/optionen/test) then 'test' else 'produktion'"/>
    
    <!-- Schlüssel für die Umkodierung lesen -->
    
    <xsl:variable name="transcode-warnings" select="($config-file-intern/res2res-config/umkodierungen/kode/@alt)"/>
    <xsl:variable name="transcode-errors" select="($config-file-intern/res2res-config/umkodierungen/kode/@neu)"/>
    
    <!-- Verarbeitung des DatML/RES Protokolls -->
    
    <xsl:template match="/">
        
        <!-- Abbruch wenn Konfigurationdatei nicht im Zugriff ist -->
        <!-- Für den Aufruf aus Java (Übergabe von Streams) delativiert
        <xsl:if test="not($cfg-exists)">
            <xsl:message terminate="yes" select="('Die Konfigurationsdatei',$config-file,'existiert nicht oder kann nicht verarbeitet werden.')"/>
        </xsl:if>
         -->
        
        <!-- Warnung wenn DatML/RAW-Dokument nicht im Zugriff ist -->
        <!-- Für den Aufruf aus Java (Übergabe von Streams) delativiert
        <xsl:if test="not($raw-exists)">
            <xsl:message select="('DatML-RAW Meldung ',$datml-raw-doc,'existiert nicht oder kann nicht verarbeitet werden.')"/>
        </xsl:if>
         -->
        
        <!-- Warnung wenn DatML/RAW-Dokument nicht im verarbeitbar ist -->
        
        <xsl:if test="not($document-processible)">
            <xsl:message select="('Die DatML-RAW Meldung',$datml-raw-doc,'ist nicht valide; es werden nur textuelle Anpassungen des Protokolls durchgeführt.')"/>
        </xsl:if>
        
        <!-- Phase 'flag' -->
        
        <xsl:variable name="with-flags-propagated">
            <xsl:apply-templates mode="flag"/>
        </xsl:variable>
        
        <!-- Phase 'message' -->
        
        <xsl:variable name="with-messages-propagated">
            <xsl:apply-templates select="$with-flags-propagated" mode="message"/>
        </xsl:variable>
        
        <!-- Phase 'document' -->
        
        <xsl:variable name="with-document-propagated">
            <xsl:apply-templates select="$with-messages-propagated" mode="document"/>
        </xsl:variable>
        
        <!-- Phase 'text' -->
        
        <xsl:apply-templates select="$with-document-propagated" mode="text"/>
        
    </xsl:template>
    
    <!-- ================ -->
    <!-- Phase 'document' -->
    <!-- ================ -->
    
    <!-- Dokumentstati setzen -->
    
    <xsl:template match="dokument" mode="document">
        <xsl:variable name="subordinateStati" select="distinct-values(nachricht/meldung/@meldungsstatus)"/>
        <xsl:variable name="mustPropagate" select="every $s in $subordinateStati satisfies $s eq 'abgewiesen'" as="xs:boolean"/>
        <xsl:element name="{local-name()}">
            <xsl:attribute name="dokumentstatus" select="if ($mustPropagate) then 'abgewiesen' else @dokumentstatus"/>
            <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else @pruefstatus"/>
            <xsl:apply-templates mode="document" select="* except (@dokumentstatus,@pruefstatus)"/>
        </xsl:element>
    </xsl:template>
    
    <!-- Nachrichtenstatus setzen -->
    
    <xsl:template match="nachricht" mode="document">
        <xsl:variable name="subordinateStati" select="distinct-values(meldung/@meldungsstatus)"/>
        <xsl:variable name="mustPropagate" select="every $s in $subordinateStati satisfies $s eq 'abgewiesen'" as="xs:boolean"/>
        <xsl:element name="{local-name()}">
            <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else @pruefstatus"/>
            <xsl:apply-templates mode="document" select="* except @pruefstatus"/>
        </xsl:element>
    </xsl:template>
    
    <!-- Attribut 'klasse' in 'absender/kennung' auf den Wert von hmm[@name='BerEmpfaenger'] setzen -->
    
    <xsl:template match="absender/kennung/@klasse" mode="document">
        <xsl:attribute name="klasse">
            <xsl:variable name="absender-klasse" select="
                if ($document-processible)
                then distinct-values(($datml-raw-doc-intern/DatML-RAW-D/nachricht//hmm[@name='BerEmpfaenger']/wert/text()))
                else ()
                "/>
            <xsl:choose>
                <xsl:when test="count($absender-klasse) gt 1">
                    <xsl:message terminate="yes">
                        ***** Dem Hilfsmerkmal [BerEmpfaenger] wurden unterschiedliche Werte zugeordnet: [<xsl:value-of select="string-join($absender-klasse,',')"/>] *****
                    </xsl:message>
                </xsl:when>
                <xsl:when test="empty($absender-klasse[1])">
                    <xsl:text>UNBEKANNT</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$absender-klasse[1]"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>
    </xsl:template>
    
    <!-- =============== -->
    <!-- Phase 'message' -->
    <!-- =============== -->
    
    <!-- Meldungsstati setzen -->
    
    <xsl:template match="meldung" mode="message">
        <xsl:variable name="codes-from-flags" select="(fehler/schluessel/text())"/>
        <xsl:variable name="mustPropagate" as="xs:boolean" select="some $w in $transcode-errors satisfies $w = $codes-from-flags"/>
        <xsl:element name="{local-name()}">
            <xsl:attribute name="meldungsstatus" select="if ($mustPropagate) then 'abgewiesen' else @meldungsstatus"/>
            <xsl:attribute name="berichtseinheitID" select="@berichtseinheitID"/>
            <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else @pruefstatus"/>
            <xsl:apply-templates mode="message" select="* except (@meldungsstatus,@pruefstatus)">
                <xsl:with-param name="mustPropagate" select="$mustPropagate" as="xs:boolean" tunnel="yes"/>
            </xsl:apply-templates>
        </xsl:element>
    </xsl:template>
    
    <!-- Status der Datenprüfung setzen -->
    
    <xsl:template match="meldung/pruefung/daten/@pruefstatus" mode="message">
        <xsl:param name="mustPropagate" as="xs:boolean" tunnel="yes"/>
        <xsl:attribute name="pruefstatus" select="if ($mustPropagate) then 'fehlerhaft' else ."/>
    </xsl:template>
    
    <!-- ============ -->
    <!-- Phase 'flag' -->
    <!-- ============ -->
    
    <!-- Fehlerschlüssel ermitteln (leer, wenn keine Ersetzung) -->
    
    <xsl:template match="fehler" mode="flag">
        <xsl:element name="fehler">
            <xsl:copy-of select="@*"/>
            <xsl:variable name="code-transform" select="$config-file-intern/res2res-config/umkodierungen/kode[@alt eq current()/schluessel/text()]"/>
            <xsl:apply-templates mode="flag">
                <xsl:with-param name="code-transform" select="$code-transform" tunnel="yes"/>
            </xsl:apply-templates>
        </xsl:element>
    </xsl:template>
    
    <!-- Fehlerschlüssel setzen -->
    
    <xsl:template match="fehler/schluessel/text()" mode="flag">
        <xsl:param name="code-transform" as="element()?" tunnel="yes" required="no"/>
        <xsl:value-of select="if (empty($code-transform)) then . else $code-transform/@neu"/>
    </xsl:template>
    
    <!-- Fehlergewicht setzen -->
    
    <xsl:template match="fehler/gewicht/text()" mode="flag">
        <xsl:param name="code-transform" as="element()?" tunnel="yes" required="no"/>
        <xsl:value-of select="if (empty($code-transform)) then . else 'ERROR'"/>
    </xsl:template>
    
    <!-- Fehlertext setzen -->
    
    <xsl:template match="fehler/text/text()" mode="flag">
        <xsl:param name="code-transform" as="element()?" tunnel="yes"/>
        <xsl:variable name="text" select="$code-transform/@text"/>
        <xsl:value-of select="if (empty($text)) then . else func:resolveText(ancestor::meldung,$text)"/>
    </xsl:template>
    
    <!-- Index von 'satz' in XPath-ausdruck durch UUID ersetzen, falls vorhanden -->
    
    <xsl:template match="fehler/position/text()" mode="flag">
        
        <xsl:variable name="xpath-mit-uuid" as="item()?">
            <xsl:if test="$document-processible and contains(.,'satz[')">
                <xsl:variable name="index-nachricht"
                    select="floor(number(substring-before(substring-after(.,'nachricht['),']')))"
                />
                <xsl:variable name="index-segment" select="if (contains(.,'segment['))
                        then floor(number(substring-before(substring-after(.,'segment['),']')))
                        else 0"
                />
                <xsl:variable name="index-satz"
                    select="floor(number(substring-before(substring-after(.,'satz['),']')))"
                />
                <xsl:variable name="uuid"
                    select="if ($index-segment gt 0)
                    then $datml-raw-doc-intern/DatML-RAW-D/nachricht[$index-nachricht]/segment[$index-segment]/datensegment/satz[$index-satz]/mm[@name='UUID'][1]/wert/text()
                    else $datml-raw-doc-intern/DatML-RAW-D/nachricht[$index-nachricht]//datensegment/satz[$index-satz]/mm[@name='UUID'][1]/wert/text()"
                />
                <!--
                <xsl:message select="$index-segment"/>
                <xsl:message select="$uuid"/>
                -->
                <xsl:if test="not(empty($uuid))">
                    <xsl:variable name="nachricht-mit-index" select="string-join(('nachricht[',string($index-nachricht),']'),'')"/>
                    <xsl:variable name="satz-mit-index" select="string-join(('satz[',string($index-satz),']'),'')"/>
                    <xsl:variable name="satz-mit-uuid" select="string-join(('satz[mm[@name=''UUID''][wert=''',string($uuid),''']]'),'')"/>
                    <xsl:variable name="pfad-bis-nachricht" select="substring-before(.,$nachricht-mit-index)"/>
                    <xsl:variable name="pfad-nachricht-satz" select="substring-after(substring-before(.,$satz-mit-index),$nachricht-mit-index)"/>
                    <xsl:variable name="pfad-satz-merkmal" select="substring-after(.,$satz-mit-index)"/>
                    <!--
                    <xsl:message select="(string-join(($nachricht-mit-index,'/',$satz-mit-index),''),':',$satz-mit-uuid),':',$pfad-satz-merkmal"/>
                    -->
                    <xsl:value-of select="string-join((
                        $pfad-bis-nachricht,
                        $nachricht-mit-index,
                        $pfad-nachricht-satz,$satz-mit-uuid,
                        $pfad-satz-merkmal
                        ),'')"
                    />
                </xsl:if>
            </xsl:if>
        </xsl:variable>
        <xsl:value-of select="if (empty($xpath-mit-uuid)) then . else $xpath-mit-uuid"/>
    </xsl:template>
    
    <!-- ============ -->
    <!-- Phase 'text' -->
    <!-- ============ -->
    
    <!-- Anwendung: Nname setzen -->
    
    <xsl:template match="dokument/dateneingang/anwendung/anwendungsname/text()" mode="text">
        <xsl:variable name="name" select="($config-file-intern/res2res-config/anwendung/name/text())"/>
        <xsl:value-of select="if (empty($name)) then . else $name"/>
    </xsl:template>
    
    <!-- Anwendung: Version setzen -->
    
    <xsl:template match="dokument/dateneingang/anwendung/version/text()" mode="text">
        <xsl:variable name="version" select="($config-file-intern/res2res-config/anwendung/version/text())"/>
        <xsl:value-of select="if (empty($version)) then . else $version"/>
    </xsl:template>
    
    <!-- Anwendung: Hersteller setzen -->
    
    <xsl:template match="dokument/dateneingang/anwendung/hersteller/text()" mode="text">
        <xsl:variable name="hersteller" select="($config-file-intern/res2res-config/anwendung/hersteller/text())"/>
        <xsl:value-of select="if (empty($hersteller)) then . else $hersteller"/>
    </xsl:template>
    
    <!-- Datenformat: Auf 'xml' setzen -->
    
    <xsl:template match="dokument/dateneingang/datei/datenformat/@typ" mode="text">
        <xsl:attribute name="typ">xml</xsl:attribute>
    </xsl:template>
    
    <!-- dokument: Hinweistext anpassen -->
    
    <xsl:template match="dokument/verarbeitungshinweis/text()" mode="text">
        <xsl:value-of select="func:getMessage('hinweis',$processingMode,../../@dokumentstatus)"/>
    </xsl:template>
    
    <!-- dokument: Erläuterungstext anpassen -->
    
    <xsl:template match="dokument/erlaeuterung/text()" mode="text">
        <xsl:value-of select="func:getMessage('erlaeuterung',$processingMode,../../@dokumentstatus)"/>
    </xsl:template>
    
    <!-- meldung: Heinweistext anpassen -->
    
    <xsl:template match="meldung/verarbeitungshinweis/text()" mode="text">
        <xsl:value-of select="func:getMessage('hinweis',$processingMode,../../@meldungsstatus)"/>
    </xsl:template>
    
    <!-- meldung: Erläuterungstext anpassen -->
    
    <xsl:template match="meldung/erlaeuterung/text()" mode="text">
        <xsl:value-of select="func:getMessage('erlaeuterung',$processingMode,../../@meldungsstatus)"/>
    </xsl:template>
    
    <!-- ========== -->
    <!-- Funktionen -->
    <!-- ========== -->
    
    <!-- Text aus Konfigurationsdatei lesen -->
    
    <xsl:function name="func:getMessage" as="xs:string">
        <xsl:param name="type" as="xs:string" required="yes"/>
        <xsl:param name="mode" as="xs:string" required="yes"/>
        <xsl:param name="status" as="xs:string" required="yes"/>
        <xsl:value-of select="($config-file-intern/res2res-config/texte[@typ eq $type][@modus eq $mode]/text[@status eq $status]/text())"/>
    </xsl:function>
    
    <!-- Variable in Text ersetzen -->
    
    <xsl:function name="func:resolveText" as="xs:string?">
        <xsl:param name="meldung" as="node()?"/>
        <xsl:param name="text" as="xs:string?"/>
        <xsl:value-of>
            <xsl:analyze-string select="$text" regex="\$\{{[^}}]+\}}">
                <xsl:matching-substring>
                    <xsl:variable name="var" select="substring-before(substring-after(.,'${'),'}')" as="xs:string"/>
                    <xsl:choose>
                        <xsl:when test="$var eq 'erhebung'">
                            <xsl:value-of select="if (empty($meldung)) then func:unresolved(.) else $meldung/erhebung/kennung"/>
                        </xsl:when>
                        <xsl:when test="$var eq 'erhebung.klasse'">
                            <xsl:value-of select="if (empty($meldung)) then func:unresolved(.) else $meldung/erhebung/kennung/@klasse"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="func:unmatched(.)"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:matching-substring>
                <xsl:non-matching-substring>
                    <xsl:value-of select="."/>
                </xsl:non-matching-substring>
            </xsl:analyze-string>
        </xsl:value-of>
    </xsl:function>
    
    <xsl:function name="func:unresolved" as="xs:string">
        <xsl:param name="varname" as="xs:string"/>
        <xsl:value-of select="concat('**KEIN WERT FUER:',$varname,'**')"/>
    </xsl:function>
    
    <xsl:function name="func:unmatched" as="xs:string">
        <xsl:param name="varname" as="xs:string"/>
        <xsl:value-of select="concat('**UNBEKANNTE VARIABLE:',$varname,'**')"/>
    </xsl:function>
    
    <!-- Zugriff auf Dokument prüfen -->
    
    <xsl:function name="func:checkDocument" as="xs:boolean">
        <xsl:param name="document" required="yes"/>
        <xsl:param name="hint" as="xs:string"/>
        <xsl:choose>
            <xsl:when test="doc-available($document)">
                <xsl:sequence select="true()"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message select="('*** Kein Zugriff auf das Dokument ''',$hint,''' an ''',$document,'''')"/>
                <xsl:sequence select="false()"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <!-- 
        Identity transform 
        ### 'mode="#all"' UND 'mode="#current"' MUESSEN GESETZT SEIN ###
    -->
    
    <xsl:template match="node()|@*" mode="#all">
        <!-- Copy the context item -->
        <xsl:copy>
            <!-- Copy any attributes and any child nodes -->
            <xsl:apply-templates select="@*|node()" mode="#current"/>
        </xsl:copy>
    </xsl:template>
    
</xsl:stylesheet>